"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Generate = void 0;
const fs_extra_1 = require("fs-extra");
const common_1 = require("./common");
class Generate extends common_1.Common {
    async exec(api, _options, args) {
        await this.setup(api.getCwd(), args);
        const typeConfig = this.config[this.fileType];
        if (this.fileType === 'component') {
            let generatableFiles = typeConfig.files;
            const templateFile = typeConfig.files.find(file => file.isTemplateFile);
            let templateFileContent;
            if (!templateFile) {
                this.log('warning', 'Template file missing, loading file content will fail.');
            }
            else {
                const tagFiles = typeConfig.files.filter(file => file.tagname);
                if (typeConfig.singleFile) {
                    templateFileContent = this.singFileTemplateContent(tagFiles);
                    generatableFiles = typeConfig.files.filter(file => file.isTemplateFile || !file.tagname);
                }
                else {
                    templateFileContent = this.multiFileTemplateContent(tagFiles);
                }
            }
            generatableFiles = generatableFiles.map(file => {
                if (file.isTemplateFile) {
                    file.content = templateFileContent;
                }
                return file;
            });
            this.generateFiles(generatableFiles);
        }
        else {
            const generatableFiles = typeConfig.files;
            this.generateFiles(generatableFiles);
        }
    }
    singFileTemplateContent(tagfiles) {
        let templateFileContent = '';
        tagfiles.forEach((file) => {
            const tagInnerContent = file.content ? file.content : '';
            const attributes = this.contructAttributes(file.tagAttributes);
            templateFileContent += `<${file.tagname}${attributes ? ` ${attributes}` : ''}>\n\t${tagInnerContent}\n</${file.tagname}> \n\n`;
        });
        return templateFileContent;
    }
    multiFileTemplateContent(tagfiles) {
        let templateFileContent = '';
        let templateFilePath;
        const templateFileObject = tagfiles.find(file => file.isTemplateFile);
        if (templateFileObject) {
            const { fileDir } = this.constructFilePath(templateFileObject);
            templateFilePath = fileDir;
        }
        tagfiles.forEach(file => {
            let tagInnerContent;
            if (file.isTemplateFile) {
                tagInnerContent = file.content ? file.content : '';
            }
            else {
                const { filePath } = this.constructFilePath(file);
                file.tagAttributes = file.tagAttributes || {};
                file.tagAttributes.src = this.getRelativePath(templateFilePath, filePath);
            }
            const attributes = this.contructAttributes(file.tagAttributes);
            templateFileContent += `<${file.tagname}${attributes ? ` ${attributes}` : ''}>${tagInnerContent ? `${tagInnerContent}` : ''}</${file.tagname}> \n\n`;
        });
        return templateFileContent;
    }
    contructAttributes(attributes) {
        return attributes ? Object.entries(attributes).map((attr) => {
            const [key, value] = attr;
            return `${key}="${value}"`;
        }).join(' ') : '';
    }
    generateFiles(files) {
        files.forEach(file => {
            const { fileDir, filePath } = this.constructFilePath(file);
            fs_extra_1.ensureDirSync(fileDir);
            if (fs_extra_1.existsSync(filePath)) {
                this.log('error', `File already exists: ${filePath}`);
            }
            else {
                fs_extra_1.writeFileSync(filePath, file.content);
                this.log('success', `Generated file: ${filePath}`);
            }
        });
    }
}
exports.Generate = Generate;
